@@ -40,6 +40,7 @@ class XFoil(ExplicitAnalysis):
                  n_crit: float = 9.,
                  xtr_upper: float = 1.,
                  xtr_lower: float = 1.,
+                 hinge_point_x: float = None,
                  full_potential: bool = False,
                  max_iter: int = 100,
                  xfoil_command: str = "xfoil",
@@ -66,6 +67,9 @@ class XFoil(ExplicitAnalysis):
 
             xtr_lower: The lower-surface trip location [x/c]
 
+            hinge_point_x: The x/c location of the hinge point. This is used to calculate hinge moment. If this is
+            None, hinge moment is not calculated.
+
             full_potential: If this is set True, it will turn full-potential mode on. Note that full-potential mode
             is only available in XFoil v7.xx or higher. (Unless you have specifically gone through the trouble of
             acquiring a copy of XFoil v7.xx you likely have v6.xx. Version 7.xx is not publicly distributed as of
@@ -114,6 +118,7 @@ class XFoil(ExplicitAnalysis):
         self.n_crit = n_crit
         self.xtr_upper = xtr_upper
         self.xtr_lower = xtr_lower
+        self.hinge_point_x = hinge_point_x
         self.full_potential = full_potential
         self.max_iter = max_iter
         self.xfoil_command = xfoil_command
@@ -123,6 +128,12 @@ class XFoil(ExplicitAnalysis):
         self.working_directory = working_directory
 
     def _default_keystrokes(self) -> List[str]:
+        """
+        Returns a list of XFoil keystrokes that are common to all XFoil runs.
+
+        Returns:
+            A list of strings, each of which is a single XFoil keystroke to be followed by <enter>.
+        """
         run_file_contents = []
 
         # Disable graphics
@@ -156,6 +167,14 @@ class XFoil(ExplicitAnalysis):
                 f"m {self.mach}",
             ]
 
+        # Handle hinge moment
+        if self.hinge_point_x is not None:
+            run_file_contents += [
+                "hinc",
+                f"fnew {self.hinge_point_x} {self.airfoil.local_camber(self.hinge_point_x)}",
+                "fmom",
+            ]
+
         if self.full_potential:
             run_file_contents += [
                 "full",
@@ -363,12 +382,12 @@ class XFoil(ExplicitAnalysis):
                     output = self._run_xfoil(
                         "\n".join(
                             [
-                                f"a {a}"
+                                f"a {a}" + ("\nfmom" if self.hinge_point_x is not None else "")
                                 for a in alphas_upper
                             ] + [
                                 "init"
                             ] + [
-                                f"a {a}"
+                                f"a {a}" + ("\nfmom" if self.hinge_point_x is not None else "")
                                 for a in alphas_lower
                             ]
                         )
@@ -383,13 +402,14 @@ class XFoil(ExplicitAnalysis):
 
         return self._run_xfoil(
             "\n".join([
-                f"a {a}"
+                f"a {a}" + ("\nfmom" if self.hinge_point_x is not None else "")
                 for a in alphas
             ])
         )
 
     def cl(self,
-           cl: Union[float, np.ndarray]
+           cl: Union[float, np.ndarray],
+           start_at: Union[float, None] = 0,
            ) -> Dict[str, np.ndarray]:
         """
         Execute XFoil at a given lift coefficient, or at a sequence of lift coefficients.
@@ -397,15 +417,54 @@ class XFoil(ExplicitAnalysis):
         Args:
             cl: The lift coefficient [-]. Can be either a float or an iterable of floats, such as an array.
 
+            start_at: Chooses whether to split a large sweep into two runs that diverge away from some central value,
+            to improve convergence. As an example, if you wanted to sweep from cl=-1.5 to cl=1.5, you might want to
+            instead do two sweeps and stitch them together: 0 to 1.5, and 0 to -1.5. `start_at` can be either:
+
+                * None, in which case the cl inputs are run as a single sequence in the order given.
+
+                * A float that corresponds to an lift coefficient, in which case the cl inputs are
+                split into two sequences that diverge from the `start_at` value. Successful runs are then sorted by
+                `alpha` before returning.
+
+
         Returns: A dictionary with the XFoil results. Dictionary values are arrays; they may not be the same shape as
         your input array if some points did not converge.
 
         """
         cls = np.array(cl).reshape(-1)
 
+        if np.length(cls) > 1:
+            if start_at is not None:
+                if np.min(cls) < start_at < np.max(cls):
+                    cls = np.sort(cls)
+                    cls_upper = cls[cls > start_at]
+                    cls_lower = cls[cls <= start_at][::-1]
+
+                    output = self._run_xfoil(
+                        "\n".join(
+                            [
+                                f"cl {c}" + ("\nfmom" if self.hinge_point_x is not None else "")
+                                for c in cls_upper
+                            ] + [
+                                "init"
+                            ] + [
+                                f"cl {c}" + ("\nfmom" if self.hinge_point_x is not None else "")
+                                for c in cls_lower
+                            ]
+                        )
+                    )
+
+                    sort_order = np.argsort(output['alpha'])
+                    output = {
+                        k: v[sort_order]
+                        for k, v in output.items()
+                    }
+                    return output
+
         return self._run_xfoil(
             "\n".join([
-                f"cl {c}"
+                f"cl {c}" + ("\nfmom" if self.hinge_point_x is not None else "")
                 for c in cls
             ])
         )
@@ -418,8 +477,9 @@ if __name__ == '__main__':
     xf = XFoil(
         airfoil=af,
         Re=1e6,
-        # verbose=True
+        hinge_point_x=0.75,
+        # verbose=True,
     )
     result_at_single_alpha = xf.alpha(5)
-    result_at_several_CLs = xf.cl([0.5, 0.7, 0.8, 0.9])
+    result_at_several_CLs = xf.cl([-0.1, 0.5, 0.7, 0.8, 0.9])
     result_at_multiple_alphas = xf.alpha([3, 5, 60])  # Note: if a result does
