@@ -52,6 +52,25 @@ class AeroBuildup(ExplicitAnalysis):
                  xyz_ref: Union[np.ndarray, List[float]] = None,
                  include_wave_drag: bool = True,
                  ):
+        """
+        Initializes a new AeroBuildup analysis as an object.
+
+        Note: to run the analysis, you need to first instantiate the object, then call the .run() method.
+
+        Args:
+            airplane: The airplane to analyze.
+
+            op_point: The operating point to analyze at. Note that this can be vectorized (i.e., attributes of the OperatingPoint
+            object can be arrays, in which case AeroBuildup analysis will be vectorized).
+
+            xyz_ref: The reference point for the aerodynamic forces and moments. This is the point about which the moments are
+            taken, and the point at which the forces are applied. Defaults to the airplane's xyz_ref.
+
+            include_wave_drag: Whether to include wave drag in the analysis. Defaults to True.
+
+        Returns: None
+
+        """
         super().__init__()
 
         ### Set defaults
@@ -64,49 +83,44 @@ class AeroBuildup(ExplicitAnalysis):
         self.xyz_ref = xyz_ref
         self.include_wave_drag = include_wave_drag
 
-    def run(self):
+    def run(self) -> Dict[str, Union[Union[float, np.ndarray], List[Union[float, np.ndarray]]]]:
         """
-        Computes the aerodynamic forces.
+        Computes the aerodynamic forces and moments on the airplane.
 
-        Returns a dictionary with keys:
+        Returns: a dictionary with keys:
 
-            'F_g' : an [x, y, z] list of forces in geometry axes [N]
+            - 'F_g' : an [x, y, z] list of forces in geometry axes [N]
+            - 'F_b' : an [x, y, z] list of forces in body axes [N]
+            - 'F_w' : an [x, y, z] list of forces in wind axes [N]
+            - 'M_g' : an [x, y, z] list of moments about geometry axes [Nm]
+            - 'M_b' : an [x, y, z] list of moments about body axes [Nm]
+            - 'M_w' : an [x, y, z] list of moments about wind axes [Nm]
+            - 'L' : the lift force [N]. Definitionally, this is in wind axes.
+            - 'Y' : the side force [N]. This is in wind axes.
+            - 'D' : the drag force [N]. Definitionally, this is in wind axes.
+            - 'l_b', the rolling moment, in body axes [Nm]. Positive is roll-right.
+            - 'm_b', the pitching moment, in body axes [Nm]. Positive is pitch-up.
+            - 'n_b', the yawing moment, in body axes [Nm]. Positive is nose-right.
+            - 'CL', the lift coefficient [-]. Definitionally, this is in wind axes.
+            - 'CY', the sideforce coefficient [-]. This is in wind axes.
+            - 'CD', the drag coefficient [-]. Definitionally, this is in wind axes.
+            - 'Cl', the rolling coefficient [-], in body axes
+            - 'Cm', the pitching coefficient [-], in body axes
+            - 'Cn', the yawing coefficient [-], in body axes
 
-            'F_b' : an [x, y, z] list of forces in body axes [N]
+            Nondimensional values are nondimensionalized using reference values in the AeroBuildup.airplane object.
 
-            'F_w' : an [x, y, z] list of forces in wind axes [N]
+            Data types:
+                - The "L", "Y", "D", "l_b", "m_b", "n_b", "CL", "CY", "CD", "Cl", "Cm", and "Cn" keys are:
 
-            'M_g' : an [x, y, z] list of moments about geometry axes [Nm]
+                    - floats if the OperatingPoint object is not vectorized (i.e., if all attributes of OperatingPoint
+                    are floats, not arrays).
 
-            'M_b' : an [x, y, z] list of moments about body axes [Nm]
+                    - arrays if the OperatingPoint object is vectorized (i.e., if any attribute of OperatingPoint is an
+                    array).
 
-            'M_w' : an [x, y, z] list of moments about wind axes [Nm]
-
-            'L' : the lift force [N]. Definitionally, this is in wind axes.
-
-            'Y' : the side force [N]. This is in wind axes.
-
-            'D' : the drag force [N]. Definitionally, this is in wind axes.
-
-            'l_b', the rolling moment, in body axes [Nm]. Positive is roll-right.
-
-            'm_b', the pitching moment, in body axes [Nm]. Positive is pitch-up.
-
-            'n_b', the yawing moment, in body axes [Nm]. Positive is nose-right.
-
-            'CL', the lift coefficient [-]. Definitionally, this is in wind axes.
-
-            'CY', the sideforce coefficient [-]. This is in wind axes.
-
-            'CD', the drag coefficient [-]. Definitionally, this is in wind axes.
-
-            'Cl', the rolling coefficient [-], in body axes
-
-            'Cm', the pitching coefficient [-], in body axes
-
-            'Cn', the yawing coefficient [-], in body axes
-
-        Nondimensional values are nondimensionalized using reference values in the AeroBuildup.airplane object.
+                - The "F_g", "F_b", "F_w", "M_g", "M_b", and "M_w" keys are always lists, which will contain either
+                floats or arrays, again depending on whether the OperatingPoint object is vectorized or not.
         """
 
         ### Compute the forces on each component
@@ -174,6 +188,67 @@ class AeroBuildup(ExplicitAnalysis):
                                        q=True,
                                        r=True,
                                        ):
+        """
+        Computes the aerodynamic forces and moments on the airplane, and the stability derivatives.
+
+        Arguments essentially determine which stability derivatives are computed. If a stability derivative is not
+        needed, leaving it False will speed up the computation.
+
+        Args:
+
+            - alpha (bool): If True, compute the stability derivatives with respect to the angle of attack (alpha).
+            - beta (bool): If True, compute the stability derivatives with respect to the sideslip angle (beta).
+            - p (bool): If True, compute the stability derivatives with respect to the body-axis roll rate (p).
+            - q (bool): If True, compute the stability derivatives with respect to the body-axis pitch rate (q).
+            - r (bool): If True, compute the stability derivatives with respect to the body-axis yaw rate (r).
+
+        Returns: a dictionary with keys:
+
+            - 'F_g' : an [x, y, z] list of forces in geometry axes [N]
+            - 'F_b' : an [x, y, z] list of forces in body axes [N]
+            - 'F_w' : an [x, y, z] list of forces in wind axes [N]
+            - 'M_g' : an [x, y, z] list of moments about geometry axes [Nm]
+            - 'M_b' : an [x, y, z] list of moments about body axes [Nm]
+            - 'M_w' : an [x, y, z] list of moments about wind axes [Nm]
+            - 'L' : the lift force [N]. Definitionally, this is in wind axes.
+            - 'Y' : the side force [N]. This is in wind axes.
+            - 'D' : the drag force [N]. Definitionally, this is in wind axes.
+            - 'l_b', the rolling moment, in body axes [Nm]. Positive is roll-right.
+            - 'm_b', the pitching moment, in body axes [Nm]. Positive is pitch-up.
+            - 'n_b', the yawing moment, in body axes [Nm]. Positive is nose-right.
+            - 'CL', the lift coefficient [-]. Definitionally, this is in wind axes.
+            - 'CY', the sideforce coefficient [-]. This is in wind axes.
+            - 'CD', the drag coefficient [-]. Definitionally, this is in wind axes.
+            - 'Cl', the rolling coefficient [-], in body axes
+            - 'Cm', the pitching coefficient [-], in body axes
+            - 'Cn', the yawing coefficient [-], in body axes
+
+            Along with additional keys, depending on the value of the `alpha`, `beta`, `p`, `q`, and `r` arguments. For
+            example, if `alpha=True`, then the following additional keys will be present:
+
+                - 'CLa', the lift coefficient derivative with respect to alpha [1/rad]
+                - 'CDa', the drag coefficient derivative with respect to alpha [1/rad]
+                - 'CYa', the sideforce coefficient derivative with respect to alpha [1/rad]
+                - 'Cla', the rolling moment coefficient derivative with respect to alpha [1/rad]
+                - 'Cma', the pitching moment coefficient derivative with respect to alpha [1/rad]
+                - 'Cna', the yawing moment coefficient derivative with respect to alpha [1/rad]
+                - 'x_np', the neutral point location in the x direction [m]
+
+            Nondimensional values are nondimensionalized using reference values in the AeroBuildup.airplane object.
+
+            Data types:
+                - The "L", "Y", "D", "l_b", "m_b", "n_b", "CL", "CY", "CD", "Cl", "Cm", and "Cn" keys are:
+
+                    - floats if the OperatingPoint object is not vectorized (i.e., if all attributes of OperatingPoint
+                    are floats, not arrays).
+
+                    - arrays if the OperatingPoint object is vectorized (i.e., if any attribute of OperatingPoint is an
+                    array).
+
+                - The "F_g", "F_b", "F_w", "M_g", "M_b", and "M_w" keys are always lists, which will contain either
+                floats or arrays, again depending on whether the OperatingPoint object is vectorized or not.
+
+        """
         abbreviations = {
             "alpha": "a",
             "beta" : "b",
@@ -281,8 +356,19 @@ class AeroBuildup(ExplicitAnalysis):
         wing_MAC = wing.mean_aerodynamic_chord()
         wing_taper = wing.taper_ratio()
         wing_sweep = wing.mean_sweep_angle()
-        AR_effective = wing.aspect_ratio(type="effective")
-        AR_geometric = wing.aspect_ratio(type="geometric")
+        wing_dihedral = wing.mean_dihedral_angle()
+
+        if wing.symmetric:
+            AR_e = wing.aspect_ratio(type="effective")
+            AR_g = wing.aspect_ratio(type="geometric")
+            AR_effective = AR_g + (AR_e - AR_g) * np.maximum(np.cosd(wing_dihedral), 0)
+            # Approximately accounts for Trefftz-pane wake continuity.
+
+            AR_geometric = AR_g
+        else:
+            AR_effective = wing.aspect_ratio(type="geometric")
+            AR_geometric = AR_effective
+
         mach = op_point.mach()
         # mach_normal = mach * np.cosd(sweep)
         AR_3D_factor = aerolib.CL_over_Cl(
@@ -307,7 +393,8 @@ class AeroBuildup(ExplicitAnalysis):
         s = np.radians(wing_sweep)
         t = np.exp(-wing_taper)
         neutral_point_deviation_due_to_unsweep = -(
-            ((((3.557726 ** (a ** 2.8443985)) * ((((s * a) + (t * 1.9149417)) + -1.4449639) * s)) + (a + -0.89228547)) * -0.16073418)
+            ((((3.557726 ** (a ** 2.8443985)) * ((((s * a) + (t * 1.9149417)) + -1.4449639) * s)) + (
+                    a + -0.89228547)) * -0.16073418)
         ) * wing_MAC
         aerodynamic_centers = [
             ac + np.array([neutral_point_deviation_due_to_unsweep, 0, 0])
@@ -318,7 +405,7 @@ class AeroBuildup(ExplicitAnalysis):
             wing._compute_xyz_of_WingXSec(
                 index=i,
                 x_nondim=0.25,
-                y_nondim=0,
+                z_nondim=0,
             )
             for i in range(len(wing.xsecs))
         ]
@@ -413,15 +500,38 @@ class AeroBuildup(ExplicitAnalysis):
             alpha_generalized_effective = (
                     alpha_generalized -
                     (1 - AR_3D_factor ** 0.8) * np.sind(2 * alpha_generalized) / 2 * (180 / np.pi)
+                # TODO: "center" this scaling around alpha = alpha_{airfoil, Cl=0}, not around alpha = 0.
+                # TODO Can estimate airfoil's alpha_{Cl=0} by camber + thin airfoil theory + viscous decambering knockdown.
             )  # Models finite-wing increase in alpha_{CL_max}.
 
             ##### Compute the control surface deflection
-            deflection = 0.
+            airfoil_a = xsec_a.airfoil
+            airfoil_b = xsec_b.airfoil
+
             for surf in xsec_a.control_surfaces:
+                try:
+                    if surf.deflection == 0:
+                        continue
+                except RuntimeError:
+                    pass
+
                 if mirror_across_XZ and not surf.symmetric:
-                    deflection -= surf.deflection
+                    deflection = -surf.deflection
                 else:
-                    deflection += surf.deflection
+                    deflection = surf.deflection
+
+                airfoil_a = airfoil_a.add_control_surface(
+                    deflection=deflection,
+                    hinge_point_x=surf.hinge_point,
+                    modify_coordinates=False,
+                    modify_polars=True,
+                )
+                airfoil_b = airfoil_b.add_control_surface(
+                    deflection=deflection,
+                    hinge_point_x=surf.hinge_point,
+                    modify_coordinates=False,
+                    modify_polars=True,
+                )
 
             ##### Compute sweep angle
             xsec_a_quarter_chord = xsec_quarter_chords[sect_id]
@@ -429,18 +539,17 @@ class AeroBuildup(ExplicitAnalysis):
             quarter_chord_vector_g = xsec_b_quarter_chord - xsec_a_quarter_chord
             quarter_chord_dir_g = quarter_chord_vector_g / np.linalg.norm(quarter_chord_vector_g)
             quarter_chord_dir_g = [  # Convert to list
-                quarter_chord_dir_g[0],
-                quarter_chord_dir_g[1],
-                quarter_chord_dir_g[2],
+                quarter_chord_dir_g[i]
+                for i in range(3)
             ]
 
-            vel_dot_quarter_chord = np.dot(
+            vel_dir_dot_quarter_chord_dir = np.dot(
                 vel_dir_g,
                 quarter_chord_dir_g,
                 manual=True
             )
 
-            sweep_rad = np.arcsin(vel_dot_quarter_chord)
+            sweep_rad = np.arcsin(vel_dir_dot_quarter_chord_dir)
 
             ##### Compute Reynolds numbers
             Re_a = op_point.reynolds(xsec_a.chord)
@@ -454,25 +563,23 @@ class AeroBuildup(ExplicitAnalysis):
                 alpha=alpha_generalized_effective,
                 Re=Re_a,
                 mach=mach_normal,
-                deflection=deflection
             )
             xsec_b_args = dict(
                 alpha=alpha_generalized_effective,
                 Re=Re_b,
                 mach=mach_normal,
-                deflection=deflection
             )
 
-            xsec_a_Cl = xsec_a.airfoil.CL_function(**xsec_a_args)
-            xsec_b_Cl = xsec_b.airfoil.CL_function(**xsec_b_args)
+            xsec_a_Cl = airfoil_a.CL_function(**xsec_a_args)
+            xsec_b_Cl = airfoil_b.CL_function(**xsec_b_args)
             sect_CL = (
                               xsec_a_Cl * a_weight +
                               xsec_b_Cl * b_weight
                       ) * AR_3D_factor ** 0.2  # Models slight decrease in finite-wing CL_max.
 
-            ##### Compute sectional drag at cross sections using lookup functions. Merge them linearly to get section CD.
-            xsec_a_Cdp = xsec_a.airfoil.CD_function(**xsec_a_args)
-            xsec_b_Cdp = xsec_b.airfoil.CD_function(**xsec_b_args)
+            ##### Compute sectional drag at cross-sections using lookup functions. Merge them linearly to get section CD.
+            xsec_a_Cdp = airfoil_a.CD_function(**xsec_a_args)
+            xsec_b_Cdp = airfoil_b.CD_function(**xsec_b_args)
             sect_CDp = (
                     (
                             xsec_a_Cdp * a_weight +
@@ -482,9 +589,9 @@ class AeroBuildup(ExplicitAnalysis):
                 # accounts for extra form factor of tip area, and 3D effects (crossflow trips)
             )
 
-            ##### Compute sectional moment at cross sections using lookup functions. Merge them linearly to get section CM.
-            xsec_a_Cm = xsec_a.airfoil.CM_function(**xsec_a_args)
-            xsec_b_Cm = xsec_b.airfoil.CM_function(**xsec_b_args)
+            ##### Compute sectional moment at cross-sections using lookup functions. Merge them linearly to get section CM.
+            xsec_a_Cm = airfoil_a.CM_function(**xsec_a_args)
+            xsec_b_Cm = airfoil_b.CM_function(**xsec_b_args)
             sect_CM = (
                     xsec_a_Cm * a_weight +
                     xsec_b_Cm * b_weight
@@ -629,8 +736,9 @@ class AeroBuildup(ExplicitAnalysis):
             xsec_b = fuselage.xsecs[sect_id + 1]
 
             ### Some metrics, like effective force location, are area-weighted. Here, we compute those weights.
-            r_a = xsec_a.radius
-            r_b = xsec_b.radius
+            r_a = xsec_a.equivalent_radius(
+                preserve="area")  # TODO modify AeroBuildup for improved accuracy on non-circular fuses
+            r_b = xsec_b.equivalent_radius(preserve="area")
 
             xyz_a = xsec_a.xyz_c
             xyz_b = xsec_b.xyz_c
@@ -766,9 +874,6 @@ class AeroBuildup(ExplicitAnalysis):
                 F_g[i] += sect_F_g[i]
                 M_g[i] += sect_M_g[i]
 
-            if fuselage.symmetric:
-                raise NotImplementedError()
-
         ##### Add in profile drag: viscous drag forces and wave drag forces
         ### Base Drag
         base_drag_coefficient = fuselage_base_drag_coefficient(mach=op_point.mach())
