@@ -358,7 +358,7 @@ class AVL(ExplicitAnalysis):
         avl_file += clean(f"""\
         {airplane.name}
         #Mach
-        0
+        0        ! AeroSandbox note: This is overwritten later to match the current OperatingPoint Mach during the AVL run.
         #IYsym   IZsym   Zsym
          0       {1 if self.ground_effect else 0}   {self.ground_effect_height}
         #Sref    Cref    Bref
@@ -541,13 +541,13 @@ class AVL(ExplicitAnalysis):
                         f"{xyz_c[0]} {xyz_c[2] + r}"
                         for xyz_c, r in zip(
                 [xsec.xyz_c for xsec in fuselage.xsecs][::-1],
-                [xsec.radius for xsec in fuselage.xsecs][::-1]
+                [xsec.equivalent_radius(preserve="area") for xsec in fuselage.xsecs][::-1]
             )
                     ] + [
                         f"{xyz_c[0]} {xyz_c[2] - r}"
                         for xyz_c, r in zip(
                 [xsec.xyz_c for xsec in fuselage.xsecs][1:],
-                [xsec.radius for xsec in fuselage.xsecs][1:]
+                [xsec.equivalent_radius(preserve="area") for xsec in fuselage.xsecs][1:]
             )
                     ]
 
