@@ -125,8 +125,6 @@ class LiftingLine(ImplicitAnalysis):
                 points, faces = wing_section.mesh_thin_surface(
                     method="quad",
                     chordwise_resolution=1,
-                    spanwise_resolution=self.spanwise_resolution,
-                    spanwise_spacing=self.spanwise_spacing,
                     add_camber=False
                 )
                 front_left_vertices.append(points[faces[:, 0], :])
@@ -1146,8 +1144,8 @@ class LiftingLine(ImplicitAnalysis):
                 xsec_1 = fuse.xsecs[xsec_id]  # type: FuselageXSec
                 xsec_2 = fuse.xsecs[xsec_id + 1]  # type: FuselageXSec
 
-                r1 = xsec_1.radius
-                r2 = xsec_2.radius
+                r1 = xsec_1.equivalent_radius(preserve="area")
+                r2 = xsec_2.equivalent_radius(preserve="area")
                 points_1 = np.zeros((fuse.circumferential_panels, 3))
                 points_2 = np.zeros((fuse.circumferential_panels, 3))
                 for point_index in range(fuse.circumferential_panels):
@@ -1170,7 +1168,6 @@ class LiftingLine(ImplicitAnalysis):
                         points_2[(point_index) % fuse.circumferential_panels, :],
                     ],
                         intensity=0,
-                        mirror=fuse.symmetric,
                     )
 
         if draw_streamlines:
