@@ -22,9 +22,9 @@ class Airfoil(Polygon):
                  name: str = "Untitled",
                  coordinates: Union[None, str, np.ndarray] = None,
                  generate_polars: bool = False,
-                 CL_function: Callable[[float, float, float, float], float] = None,
-                 CD_function: Callable[[float, float, float, float], float] = None,
-                 CM_function: Callable[[float, float, float, float], float] = None,
+                 CL_function: Callable[[float, float, float], float] = None,
+                 CD_function: Callable[[float, float, float], float] = None,
+                 CM_function: Callable[[float, float, float], float] = None,
                  ):
         """
         Creates an Airfoil object.
@@ -68,7 +68,7 @@ class Airfoil(Polygon):
 
                 Must be a callable with that takes exactly these parameters as follows:
 
-                >>> def CL_function(alpha, Re, mach, deflection)
+                >>> def CL_function(alpha, Re, mach)
 
                 where:
 
@@ -78,9 +78,6 @@ class Airfoil(Polygon):
 
                     * `mach` is the local Mach number
 
-                    * `deflection` is the deflection of any control surface on the airfoil, given in degrees. Following
-                    convention, a positive control surface deflections corresponds to a downwards deflection of a trailing-edge surface.
-
             CD_function: A function that gives the sectional drag coefficient of the airfoil as a function of
             several parameters.
 
@@ -147,9 +144,35 @@ class Airfoil(Polygon):
         if CM_function is not None:
             self.CM_function = CM_function
 
-    def __repr__(self):  # String representation
+    def __repr__(self) -> str:
         return f"Airfoil {self.name} ({self.n_points()} points)"
 
+    def __eq__(self, other: "Airfoil") -> bool:
+        """
+        Checks if two airfoils are equal. Two airfoils are equal if they have the same name, coordinates, and
+        polar functions.
+
+        Args:
+            other: The other airfoil to compare to.
+
+        Returns:
+            True if the two airfoils are equal, False otherwise.
+        """
+        if other is self:  # If they're the same object in memory, they're equal
+            return True
+
+        if not type(self) == type(other):  # If the types are different, they're not equal
+            return False
+
+        # At this point, we know that the types are the same, so we can compare the attributes
+        return all([  # If all of these are true, they're equal
+            self.name == other.name,
+            np.allclose(self.coordinates, other.coordinates),
+            self.CL_function is other.CL_function,
+            self.CD_function is other.CD_function,
+            self.CM_function is other.CM_function,
+        ])
+
     def generate_polars(self,
                         alphas=np.linspace(-15, 15, 21),
                         Res=np.geomspace(1e4, 1e7, 10),
@@ -159,18 +182,17 @@ class Airfoil(Polygon):
                         include_compressibility_effects: bool = True,
                         transonic_buffet_lift_knockdown: float = 0.3,
                         make_symmetric_polars: bool = False,
-                        add_deflections_as_all_moving_surfaces: bool = True,
                         ) -> None:
         """
         Generates airfoil polar surrogate models (CL, CD, CM functions) from XFoil data and assigns them in-place to
         this Airfoil's polar functions.
 
         In other words, when this function is run, the following functions will be added (or overwritten) to the instance:
-            * Airfoil.CL_function(alpha, Re, mach, deflection)
-            * Airfoil.CD_function(alpha, Re, mach, deflection)
-            * Airfoil.CM_function(alpha, Re, mach, deflection)
+            * Airfoil.CL_function(alpha, Re, mach)
+            * Airfoil.CD_function(alpha, Re, mach)
+            * Airfoil.CM_function(alpha, Re, mach)
 
-        Where alpha is in degrees. Right now, deflection is not used.
+        Where alpha is in degrees.
 
         Args:
 
@@ -196,11 +218,6 @@ class Airfoil(Polygon):
             mach tuck, CL effects across normal shocks. Note that accuracy here is dubious in the transonic regime
             and above - you should really specify your own CL/CD/CM models
 
-            add_deflections_as_all_moving_surfaces: If this boolean is flagged `True`, the `deflection=` keyword
-            argument of the generated polar functions directly modifies the angle of attack. This effectively means
-            that you are assuming an all-moving airfoil hinged about the quarter-chord. If this boolean is flagged
-            `False`, the `deflection=` keyword argument will do nothing.
-
         Warning: In-place operation! Modifies this Airfoil object by setting Airfoil.CL_function, etc. to the new
         polars.
 
@@ -378,9 +395,7 @@ class Airfoil(Polygon):
             y_data=CM_if_separated
         )
 
-        def CL_function(alpha, Re, mach=0, deflection=0):
-            if add_deflections_as_all_moving_surfaces:
-                alpha = alpha + deflection
+        def CL_function(alpha, Re, mach=0):
 
             alpha = np.mod(alpha + 180, 360) - 180  # Keep alpha in the valid range.
             CL_attached = CL_attached_interpolator({
@@ -433,9 +448,7 @@ class Airfoil(Polygon):
             else:
                 return CL_mach_0
 
-        def CD_function(alpha, Re, mach=0, deflection=0):
-            if add_deflections_as_all_moving_surfaces:
-                alpha = alpha + deflection
+        def CD_function(alpha, Re, mach=0):
 
             alpha = np.mod(alpha + 180, 360) - 180  # Keep alpha in the valid range.
             log10_CD_attached = log10_CD_attached_interpolator({
@@ -476,7 +489,6 @@ class Airfoil(Polygon):
                 t_over_c = self.max_thickness()
 
                 mach_crit = transonic.mach_crit_Korn(
-                    # CL=CL_function(alpha, Re, mach, deflection),
                     CL=CL,
                     t_over_c=t_over_c,
                     sweep=0,
@@ -533,9 +545,7 @@ class Airfoil(Polygon):
             else:
                 return 10 ** log10_CD_mach_0
 
-        def CM_function(alpha, Re, mach=0, deflection=0):
-            if add_deflections_as_all_moving_surfaces:
-                alpha = alpha + deflection
+        def CM_function(alpha, Re, mach=0):
 
             alpha = np.mod(alpha + 180, 360) - 180  # Keep alpha in the valid range.
             CM_attached = CM_attached_interpolator({
@@ -573,8 +583,12 @@ class Airfoil(Polygon):
                      ) -> Union[float, np.ndarray]:
         """
         Returns the local camber of the airfoil at a given point or points.
-        :param x_over_c: The x/c locations to calculate the camber at [1D array, more generally, an iterable of floats]
-        :return: Local camber of the airfoil (y/c) [1D array].
+
+        Args:
+            x_over_c: The x/c locations to calculate the camber at [1D array, more generally, an iterable of floats]
+
+        Returns:
+            Local camber of the airfoil (y/c) [1D array].
         """
         upper = self.upper_coordinates()[::-1]
         lower = self.lower_coordinates()
@@ -597,8 +611,12 @@ class Airfoil(Polygon):
                         ) -> Union[float, np.ndarray]:
         """
         Returns the local thickness of the airfoil at a given point or points.
-        :param x_over_c: The x/c locations to calculate the thickness at [1D array, more generally, an iterable of floats]
-        :return: Local thickness of the airfoil (y/c) [1D array].
+
+        Args:
+            x_over_c: The x/c locations to calculate the thickness at [1D array, more generally, an iterable of floats]
+
+        Returns:
+            Local thickness of the airfoil (y/c) [1D array].
         """
         upper = self.upper_coordinates()[::-1]
         lower = self.lower_coordinates()
@@ -648,12 +666,18 @@ class Airfoil(Polygon):
              draw_mcl=True,
              backend="matplotlib",
              show=True
-             ):
+             ) -> None:
         """
         Draw the airfoil object.
-        :param draw_mcl: Should we draw the mean camber line (MCL)? [boolean]
-        :param backend: Which backend should we use? "plotly" or "matplotlib"
-        :return: None
+
+        Args:
+            draw_mcl: Should we draw the mean camber line (MCL)? [boolean]
+
+            backend: Which backend should we use? "plotly" or "matplotlib"
+
+            show: Should we show the plot? [boolean]
+
+        Returns: None
         """
         x = np.array(self.x()).reshape(-1)
         y = np.array(self.y()).reshape(-1)
@@ -716,7 +740,7 @@ class Airfoil(Polygon):
 
     def LE_index(self) -> int:
         """
-        Returns the index of the leading-edge point.
+        Returns the index of the leading edge point in the airfoil coordinates.
         """
         return np.argmin(self.x())
 
@@ -746,7 +770,13 @@ class Airfoil(Polygon):
         """
         Returns the thickness of the trailing edge of the airfoil.
         """
-        return self.local_thickness(x_over_c=1)
+        x_gap = self.coordinates[0, 0] - self.coordinates[-1, 0]
+        y_gap = self.coordinates[0, 1] - self.coordinates[-1, 1]
+
+        return (
+                x_gap ** 2 +
+                y_gap ** 2
+        ) ** 0.5
 
     def TE_angle(self) -> float:
         """
@@ -755,10 +785,10 @@ class Airfoil(Polygon):
         upper_TE_vec = self.coordinates[0, :] - self.coordinates[1, :]
         lower_TE_vec = self.coordinates[-1, :] - self.coordinates[-2, :]
 
-        return 180 / np.pi * (np.arctan2(
+        return np.arctan2d(
             upper_TE_vec[0] * lower_TE_vec[1] - upper_TE_vec[1] * lower_TE_vec[0],
             upper_TE_vec[0] * lower_TE_vec[0] + upper_TE_vec[1] * upper_TE_vec[1]
-        ))
+        )
 
     # def LE_radius(self) -> float:
     #     """
@@ -767,13 +797,23 @@ class Airfoil(Polygon):
 
     def repanel(self,
                 n_points_per_side: int = 100,
+                spacing_function_per_side = np.cosspace,
                 ) -> 'Airfoil':
         """
-        Returns a repaneled version of the airfoil with cosine-spaced coordinates on the upper and lower surfaces.
-        :param n_points_per_side: Number of points per side (upper and lower) of the airfoil [int]
-            Notes: The number of points defining the final airfoil will be n_points_per_side*2-1,
-            since one point (the leading edge point) is shared by both the upper and lower surfaces.
-        :return: Returns the new airfoil.
+        Returns a repaneled copy of the airfoil with cosine-spaced coordinates on the upper and lower surfaces.
+
+        Args:
+
+            n_points_per_side: Number of points per side (upper and lower) of the airfoil [int]
+
+                Notes: The number of points defining the final airfoil will be `n_points_per_side * 2 - 1`,
+                since one point (the leading edge point) is shared by both the upper and lower surfaces.
+
+            spacing_function_per_side: Determines how to space the points on each side of the airfoil. Can be
+            `np.linspace` or `np.cosspace`, or any other function of the call signature `f(a, b, n)` that returns
+            a spaced array of `n` points between `a` and `b`. [function]
+
+        Returns: A copy of the airfoil with the new coordinates.
         """
 
         upper_original_coors = self.upper_coordinates()  # Note: includes leading edge point, be careful about duplicates
@@ -799,10 +839,10 @@ class Airfoil(Polygon):
         ))
 
         # Generate a cosine-spaced list of points from 0 to 1
-        cosspaced_points = np.cosspace(0, 1, n_points_per_side)
+        spaced_points = spacing_function_per_side(0, 1, n_points_per_side)
         s = np.hstack((
-            cosspaced_points,
-            1 + cosspaced_points[1:],
+            spaced_points,
+            1 + spaced_points[1:],
         ))
 
         # Check that there are no duplicate points in the airfoil.
@@ -831,36 +871,206 @@ class Airfoil(Polygon):
             self,
             deflection: float = 0.,
             hinge_point_x: float = 0.75,
+            modify_coordinates: bool = True,
+            modify_polars: bool = True,
     ) -> 'Airfoil':
         """
-        Returns a version of the airfoil with a control surface added at a given point. Implicitly repanels the airfoil as part of this operation.
-        :param deflection: deflection angle [degrees]. Downwards-positive.
-        :param hinge_point_x: location of the hinge, as a fraction of chord [float].
-        :return: The new airfoil.
+        Returns a version of the airfoil with a trailing-edge control surface added at a given point. Implicitly
+        repanels the airfoil as part of this operation.
+
+        Args:
+            deflection: Deflection angle [degrees]. Downwards-positive.
+            hinge_point_x: Chordwise location of the hinge, as a fraction of chord (x/c) [float]
+
+        Returns: an Airfoil object with the new control deflection.
+
         """
+        if modify_coordinates:
+            # Find the hinge point
+            hinge_point_y = np.where(
+                deflection > 0,
+                self.local_camber(hinge_point_x) - self.local_thickness(hinge_point_x) / 2,
+                self.local_camber(hinge_point_x) + self.local_thickness(hinge_point_x) / 2,
+            )
 
-        # Make the rotation matrix for the given angle.
-        rotation_matrix = np.rotations.rotation_matrix_2D(-np.pi / 180 * deflection)
+            # hinge_point_y = self.local_camber(hinge_point_x)
+            hinge_point = np.reshape(
+                np.array([hinge_point_x, hinge_point_y]),
+                (1, 2)
+            )
 
-        # Find the hinge point
-        hinge_point_y = self.local_camber(hinge_point_x)
+            def is_behind_hinge(xy: np.ndarray) -> np.ndarray:
+                return (
+                        (xy[:, 0] - hinge_point_x) * np.cosd(deflection / 2) -
+                        (xy[:, 1] - hinge_point_y) * np.sind(deflection / 2)
+                        > 0
+                )
 
-        # Find the coordinates of a rotated airfoil
-        rotated_airfoil = self.rotate(
-            angle=-np.pi / 180 * deflection,
-            x_center=hinge_point_x,
-            y_center=hinge_point_y,
-        )
+            orig_u = self.upper_coordinates()
+            orig_l = self.lower_coordinates()
 
-        # Merge the two sets of coordinates
+            rotation_matrix = np.rotation_matrix_2D(
+                angle=-np.radians(deflection),
+            )
 
-        coordinates = np.copy(self.coordinates)
-        is_past_hinge = self.x() > hinge_point_x  # TODO fix hinge self-intersecting paneling issue for large deflection
-        coordinates[is_past_hinge] = rotated_airfoil.coordinates[is_past_hinge]
+            def T(xy):
+                return np.transpose(xy)
+
+            hinge_point_u = np.tile(hinge_point, (np.length(orig_u), 1))
+            hinge_point_l = np.tile(hinge_point, (np.length(orig_l), 1))
+
+            rot_u = T(rotation_matrix @ T(orig_u - hinge_point_u)) + hinge_point_u
+            rot_l = T(rotation_matrix @ T(orig_l - hinge_point_l)) + hinge_point_l
+
+            coordinates_x = np.concatenate([
+                np.where(
+                    is_behind_hinge(rot_u),
+                    rot_u[:, 0],
+                    orig_u[:, 0]
+                ),
+                np.where(
+                    is_behind_hinge(rot_l),
+                    rot_l[:, 0],
+                    orig_l[:, 0]
+                )
+            ])
+            coordinates_y = np.concatenate([
+                np.where(
+                    is_behind_hinge(rot_u),
+                    rot_u[:, 1],
+                    orig_u[:, 1]
+                ),
+                np.where(
+                    is_behind_hinge(rot_l),
+                    rot_l[:, 1],
+                    orig_l[:, 1]
+                )
+            ])
+
+            coordinates = np.stack([
+                coordinates_x,
+                coordinates_y
+            ], axis=1)
+        else:
+            coordinates = self.coordinates
+
+        if modify_polars:
+            effectiveness = 1 - np.maximum(0, hinge_point_x + 1e-16) ** 2.751428551177291
+            dalpha = deflection * effectiveness
+
+            def CL_function(alpha: float, Re: float, mach: float) -> float:
+                return self.CL_function(
+                    alpha=alpha + dalpha,
+                    Re=Re,
+                    mach=mach,
+                )
+
+            def CD_function(alpha: float, Re: float, mach: float) -> float:
+                return self.CD_function(
+                    alpha=alpha + dalpha,
+                    Re=Re,
+                    mach=mach,
+                )
+
+            def CM_function(alpha: float, Re: float, mach: float) -> float:
+                return self.CM_function(
+                    alpha=alpha + dalpha,
+                    Re=Re,
+                    mach=mach,
+                )
+
+        else:
+            CL_function = self.CL_function
+            CD_function = self.CD_function
+            CM_function = self.CM_function
 
         return Airfoil(
             name=self.name,
-            coordinates=coordinates
+            coordinates=coordinates,
+            CL_function=CL_function,
+            CD_function=CD_function,
+            CM_function=CM_function,
+        )
+
+    def set_TE_thickness(self,
+                         thickness: float = 0.,
+                         ) -> 'Airfoil':
+        """
+        Creates a modified copy of the Airfoil that has a specified trailing-edge thickness.
+
+        Note that the trailing-edge thickness is given nondimensionally (e.g., as a fraction of chord).
+
+        Args:
+            thickness: The target trailing-edge thickness, given nondimensionally (e.g., as a fraction of chord).
+
+        Returns: The modified airfoil.
+
+        """
+        ### Compute existing trailing-edge properties
+        x_gap = self.coordinates[0, 0] - self.coordinates[-1, 0]
+        y_gap = self.coordinates[0, 1] - self.coordinates[-1, 1]
+
+        s_gap = (
+                        x_gap ** 2 +
+                        y_gap ** 2
+                ) ** 0.5
+
+        s_adjustment = (thickness - self.TE_thickness()) / 2
+
+        ### Determine how much the trailing edge should move by in X and Y.
+        if s_gap != 0:
+            x_adjustment = s_adjustment * x_gap / s_gap
+            y_adjustment = s_adjustment * y_gap / s_gap
+        else:
+            x_adjustment = 0
+            y_adjustment = s_adjustment
+
+        ### Decompose the existing airfoil coordinates to upper and lower sides, and x and y.
+        u = self.upper_coordinates()
+        ux = u[:, 0]
+        uy = u[:, 1]
+
+        le_x = ux[-1]
+
+        l = self.lower_coordinates()[1:]
+        lx = l[:, 0]
+        ly = l[:, 1]
+
+        te_x = (ux[0] + lx[-1]) / 2
+
+        ### Create modified versions of the upper and lower coordinates
+        new_u = np.stack(
+            arrays=[
+                ux + x_adjustment * (ux - le_x) / (te_x - le_x),
+                uy + y_adjustment * (ux - le_x) / (te_x - le_x)
+            ],
+            axis=1
+        )
+        new_l = np.stack(
+            arrays=[
+                lx - x_adjustment * (lx - le_x) / (te_x - le_x),
+                ly - y_adjustment * (lx - le_x) / (te_x - le_x)
+            ],
+            axis=1
+        )
+
+        ### If the desired thickness is zero, ensure that is precisely reached.
+        if thickness == 0:
+            new_l[-1] = new_u[0]
+
+        ### Combine the upper and lower surface coordinates into a single array.
+        new_coordinates = np.concatenate(
+            [
+                new_u,
+                new_l
+            ],
+            axis=0
+        )
+
+        ### Return a new Airfoil with the desired coordinates.
+        return Airfoil(
+            name=self.name,
+            coordinates=new_coordinates
         )
 
     def scale(self,
@@ -984,22 +1194,22 @@ class Airfoil(Polygon):
                 that_fraction * that_foil.coordinates
         )
 
-        def CL_function(alpha, Re, mach, deflection):
+        def CL_function(alpha, Re, mach):
             return (
-                    this_fraction * this_foil.CL_function(alpha, Re, mach, deflection) +
-                    that_fraction * that_foil.CL_function(alpha, Re, mach, deflection)
+                    this_fraction * this_foil.CL_function(alpha, Re, mach) +
+                    that_fraction * that_foil.CL_function(alpha, Re, mach)
             )
 
-        def CD_function(alpha, Re, mach, deflection):
+        def CD_function(alpha, Re, mach):
             return (
-                    this_fraction * this_foil.CD_function(alpha, Re, mach, deflection) +
-                    that_fraction * that_foil.CD_function(alpha, Re, mach, deflection)
+                    this_fraction * this_foil.CD_function(alpha, Re, mach) +
+                    that_fraction * that_foil.CD_function(alpha, Re, mach)
             )
 
-        def CM_function(alpha, Re, mach, deflection):
+        def CM_function(alpha, Re, mach):
             return (
-                    this_fraction * this_foil.CM_function(alpha, Re, mach, deflection) +
-                    that_fraction * that_foil.CM_function(alpha, Re, mach, deflection)
+                    this_fraction * this_foil.CM_function(alpha, Re, mach) +
+                    that_fraction * that_foil.CM_function(alpha, Re, mach)
             )
 
         return Airfoil(
@@ -1431,3 +1641,8 @@ class Airfoil(Polygon):
     #     plt.show()
     #
     #     return self
+
+
+if __name__ == '__main__':
+    af = Airfoil("naca2412")
+    af.add_control_surface(45).draw()
