@@ -2,7 +2,7 @@ import aerosandbox.numpy as np
 from typing import Tuple
 
 """
-Documentation of (points, faces) standard format:
+Documentation of (points, faces) standard format, which is an unstructured mesh format:
 
 Meshes are given here in the common (points, faces) format. In this format, `points` is a Nx3 array, where each row 
 gives the 3D coordinates of a vertex in the mesh. Entries into this array are floating-point, generally speaking.
@@ -15,8 +15,8 @@ the vertex locations of that face are found.
 
 
 def stack_meshes(
-        *meshes: Tuple[np.ndarray, np.ndarray]
-):
+        *meshes: Tuple[Tuple[np.ndarray, np.ndarray]]
+) -> Tuple[np.ndarray, np.ndarray]:
     """
     Takes in a series of tuples (points, faces) and merges them into a single tuple (points, faces). All (points,
     faces) tuples are meshes given in standard format.
@@ -24,7 +24,14 @@ def stack_meshes(
     Args:
         *meshes: Any number of mesh tuples in standard (points, faces) format.
 
-    Returns: (points, faces) of the combined mesh.
+    Returns: Points and faces of the combined mesh. Standard unstructured mesh format: A tuple of `points` and
+    `faces`, where:
+
+        * `points` is a `n x 3` array of points, where `n` is the number of points in the mesh.
+
+        * `faces` is a `m x 3` array of faces if `method` is "tri", or a `m x 4` array of faces if `method` is "quad".
+
+            * Each row of `faces` is a list of indices into `points`, which specifies a face.
 
     """
     if len(meshes) == 1:
@@ -53,7 +60,7 @@ def stack_meshes(
 def convert_mesh_to_polydata_format(
         points: np.ndarray,
         faces: np.ndarray
-):
+) -> Tuple[np.ndarray, np.ndarray]:
     """
     PyVista uses a slightly different convention for the standard (points, faces) format as described above. They
     give `faces` as a single 1D vector of roughly length (M*3), or (M*4) in the case of quadrilateral meshing.
