@@ -11,21 +11,24 @@ class Polygon(AeroSandboxObject):
     def x(self) -> np.ndarray:
         """
         Returns the x coordinates of the polygon. Equivalent to Polygon.coordinates[:,0].
-        :return: X coordinates as a vector
+
+        Returns:
+            X coordinates as a vector
         """
         return self.coordinates[:, 0]
 
     def y(self) -> np.ndarray:
         """
         Returns the y coordinates of the polygon. Equivalent to Polygon.coordinates[:,1].
-        :return: Y coordinates as a vector
+
+        Returns:
+            Y coordinates as a vector
         """
         return self.coordinates[:, 1]
 
     def n_points(self) -> int:
         """
         Returns the number of points/vertices/coordinates of the polygon.
-        Analogous to len(coordinates)
         """
         try:
             return len(self.coordinates)
@@ -40,13 +43,16 @@ class Polygon(AeroSandboxObject):
                         y: Union[float, np.ndarray],
                         ) -> np.ndarray:
         """
-        Returns a boolean array of whether or not some (x, y) point(s) are contained within the Polygon.
+        Returns a boolean array of whether some (x, y) point(s) are contained within the Polygon.
 
         Args:
             x: x-coordinate(s) of the query points.
             y: y-coordinate(s) of the query points.
 
-        Returns: A boolean array of the same size as x and y.
+        Returns:
+
+            A boolean array of the same size as x and y, with values corresponding to whether or not the points are
+            inside the Polygon.
 
         """
         x = np.array(x)
@@ -85,6 +91,19 @@ class Polygon(AeroSandboxObject):
 
         return A
 
+    def perimeter(self) -> float:
+        """
+        Returns the perimeter of the polygon.
+        """
+        dx = np.diff(self.x())
+        dy = np.diff(self.y())
+        ds = (
+                     dx ** 2 +
+                     dy ** 2
+             ) ** 0.5
+
+        return np.sum(ds)
+
     def centroid(self) -> np.ndarray:
         """
         Returns the centroid of the polygon as a 1D np.ndarray of length 2.
@@ -212,4 +231,4 @@ def stack_coordinates(
     Returns: A Nx2 ndarray of [x, y] coordinates.
 
     """
-    return np.vstack((x, y)).T
+    return np.stack((x, y), axis=1)
