@@ -1,29 +1,61 @@
 import aerosandbox.numpy as np
+from aerosandbox.tools import units as u
+from typing import Union, Dict
 
 
 def motor_electric_performance(
-        voltage=None,
-        current=None,
-        rpm=None,
-        torque=None,
-        kv=1000,  # rpm/volt
-        resistance=0.1,  # ohms
-        no_load_current=0.4  # amps
-):
+        voltage: Union[float, np.ndarray] = None,
+        current: Union[float, np.ndarray] = None,
+        rpm: Union[float, np.ndarray] = None,
+        torque: Union[float, np.ndarray] = None,
+        kv: float = 1000.,  # rpm/volt
+        resistance: float = 0.1,  # ohms
+        no_load_current: float = 0.4  # amps
+) -> Dict[str, Union[float, np.ndarray]]:
     """
     A function for predicting the performance of an electric motor.
+    
     Performance equations based on Mark Drela's First Order Motor Model:
     http://web.mit.edu/drela/Public/web/qprop/motor1_theory.pdf
+    
     Instructions: Input EXACTLY TWO of the following parameters: voltage, current, rpm, torque.
+    
     Exception: You cannot supply the combination of current and torque - this makes for an ill-posed problem.
-    :param voltage: Voltage across motor terminals [Volts]
-    :param current: Current through motor [Amps]
-    :param rpm: Motor rotation speed [rpm]
-    :param torque: Motor torque [N m]
-    :param kv: voltage constant, in rpm/volt
-    :param resistance: resistance, in ohms
-    :param no_load_current: no-load current, in amps
-    :return: dict of {voltage, current, rpm, torque, efficiency}
+    
+    Note that this function is fully vectorized, so arrays can be supplied to any of the inputs.
+    
+    Args:
+        voltage: Voltage across motor terminals [Volts]
+        
+        current: Current through motor [Amps]
+        
+        rpm: Motor rotation speed [rpm]
+        
+        torque: Motor torque [N m]
+        
+        kv: voltage constant, in rpm/volt
+        
+        resistance: resistance, in ohms
+        
+        no_load_current: no-load current, in amps
+        
+    Returns:
+        A dictionary where keys are: 
+            "voltage", 
+            "current", 
+            "rpm", 
+            "torque",
+            "shaft power", 
+            "electrical power", 
+            "efficiency"
+            "waste heat"
+
+        And values are corresponding quantities in SI units.
+
+        Note that "efficiency" is just (shaft power) / (electrical power), and hence implicitly assumes that the
+        motor is operating as a motor (electrical -> shaft power), and not a generator (shaft power -> electrical).
+        If you want to know the efficiency of the motor as a generator, you can simply calculate it as (electrical
+        power) / (shaft power).
     """
     # Validate inputs
     voltage_known = voltage is not None
@@ -31,13 +63,18 @@ def motor_electric_performance(
     rpm_known = rpm is not None
     torque_known = torque is not None
 
-    assert (
-                   voltage_known + current_known + rpm_known + torque_known) == 2, "You must give exactly two input arguments."
-    assert not (
-            current_known and torque_known), "You cannot supply the combination of current and torque - this makes for an ill-posed problem."
+    if not (
+            voltage_known + current_known + rpm_known + torque_known == 2
+    ):
+        raise ValueError("You must give exactly two input arguments.")
+
+    if current_known and torque_known:
+        raise ValueError(
+            "You cannot supply the combination of current and torque - this makes for an ill-posed problem.")
 
     kv_rads_per_sec_per_volt = kv * np.pi / 30  # rads/sec/volt
 
+    ### Iterate through the motor equations until all quantities are known.
     while not (voltage_known and current_known and rpm_known and torque_known):
         if rpm_known:
             if current_known and not voltage_known:
@@ -90,10 +127,15 @@ def motor_resistance_from_no_load_current(
 ):
     """
     Estimates the internal resistance of a motor from its no_load_current. Gates quotes R^2=0.93 for this model.
+
     Source: Gates, et. al., "Combined Trajectory, Propulsion, and Battery Mass Optimization for Solar-Regen..."
         https://scholarsarchive.byu.edu/cgi/viewcontent.cgi?article=3932&context=facpub
-    :param no_load_current: No-load current [amps]
-    :return: motor internal resistance [ohms]
+
+    Args:
+        no_load_current: No-load current [amps]
+
+    Returns:
+        motor internal resistance [ohms]
     """
     return 0.0467 * no_load_current ** -1.892
 
@@ -103,9 +145,14 @@ def mass_ESC(
 ):
     """
     Estimates the mass of an ESC.
-    Informal correlation I did to Hobbyking ESCs in the 8s 100A range
-    :param max_power: maximum power [W]
-    :return: estimated ESC mass [kg]
+
+    Informal correlation I did to Hobbyking ESCs in the 8S LiPo, 100A range
+
+    Args:
+        max_power: maximum power [W]
+
+    Returns:
+        estimated ESC mass [kg]
     """
     return 2.38e-5 * max_power
 
@@ -113,14 +160,23 @@ def mass_ESC(
 def mass_battery_pack(
         battery_capacity_Wh,
         battery_cell_specific_energy_Wh_kg=240,
-        battery_pack_cell_fraction=0.7,  # Figure given by Ed Lovelace in a Feb. 2020 presentation for MIT 16.82
+        battery_pack_cell_fraction=0.7,
 ):
     """
     Estimates the mass of a lithium-polymer battery.
-    :param battery_capacity_Wh: Battery capacity, in Watt-hours [W*h]
-    :param battery_cell_specific_energy: Specific energy of the battery at the CELL level [W*h/kg]
-    :param battery_pack_cell_fraction: Fraction of the battery pack that is cells, by weight.
-    :return: Estimated battery mass [kg]
+
+    Args:
+        battery_capacity_Wh: Battery capacity, in Watt-hours [W*h]
+
+        battery_cell_specific_energy: Specific energy of the battery at the CELL level [W*h/kg]
+
+        battery_pack_cell_fraction: Fraction of the battery pack that is cells, by weight.
+
+            * Note: Ed Lovelace, a battery engineer for Aurora Flight Sciences, gives this figure as 0.70 in a Feb.
+            2020 presentation for MIT 16.82
+
+    Returns:
+        Estimated battery mass [kg]
     """
     return battery_capacity_Wh / battery_cell_specific_energy_Wh_kg / battery_pack_cell_fraction
 
@@ -135,16 +191,27 @@ def mass_motor_electric(
     Estimates the mass of a brushless DC electric motor.
     Curve fit to scraped Hobbyking BLDC motor data as of 2/24/2020.
     Estimated range of validity: 50 < max_power < 10000
-    :param max_power: maximum power [W]
-    :param kv: Voltage constant of the motor, measured in rpm/volt, not rads/sec/volt! [rpm/volt]
-    :param voltage: Operating voltage of the motor [V]
-    :param method: method to use. "burton", "hobbyking", or "astroflight" (increasing level of detail).
-    Burton source: https://dspace.mit.edu/handle/1721.1/112414
-    Hobbyking source: C:\Projects\GitHub\MotorScraper, https://github.com/austinstover/MotorScraper
-    Astroflight source: Gates, et. al., "Combined Trajectory, Propulsion, and Battery Mass Optimization for Solar-Regen..."
-        https://scholarsarchive.byu.edu/cgi/viewcontent.cgi?article=3932&context=facpub
-        Validity claimed from 1.5 kW to 15 kW, kv from 32 to 1355.
-    :return: estimated motor mass [kg]
+
+    Args:
+        max_power (float): maximum power [W]
+
+        kv_rpm_volt (float): Voltage constant of the motor, measured in rpm/volt, not rads/sec/volt! [rpm/volt]
+
+        voltage (float): Operating voltage of the motor [V]
+
+        method (str): method to use. "burton", "hobbyking", or "astroflight" (increasing level of detail).
+
+            * Burton source: https://dspace.mit.edu/handle/1721.1/112414
+
+            * Hobbyking source: C:\Projects\GitHub\MotorScraper,
+
+            * Astroflight source: Gates, et. al., "Combined Trajectory, Propulsion, and Battery Mass Optimization for
+            Solar-Regen..." https://scholarsarchive.byu.edu/cgi/viewcontent.cgi?article=3932&context=facpub
+
+                * Validity claimed from 1.5 kW to 15 kW, kv from 32 to 1355.
+
+    Returns:
+        Estimated motor mass [kg]
     """
     if method == "burton":
         return max_power / 4128  # Less sophisticated model. 95% CI (3992, 4263), R^2 = 0.866
@@ -164,22 +231,68 @@ def mass_wires(
         max_voltage=600,
         wire_packing_factor=1,
         insulator_density=1700,
-        insulator_dielectric_strength=12e6
+        insulator_dielectric_strength=12e6,
+        insulator_min_thickness=0.2e-3,  # silicone wire
+        return_dict: bool = False
 ):
     """
     Estimates the mass of wires used for power transmission.
+
     Materials data from: https://en.wikipedia.org/wiki/Electrical_resistivity_and_conductivity#Resistivity-density_product
         All data measured at STP; beware, as this data (especially resistivity) can be a strong function of temperature.
-    :param wire_length: Length of the wire [m]
-    :param max_current: Max current of the wire [Amps]
-    :param allowable_voltage_drop: How much is the voltage allowed to drop along the wire?
-    :param material: Conductive material of the wire ("aluminum")
-    :param insulated: Should we add the mass of the wire's insulator coating? Usually you'll want to leave this True.
-    :param max_voltage: Maximum allowable voltage (used for sizing insulator). 600 is a common off-the-shelf rating.
-    :param wire_packing_factor: What fraction of the enclosed cross section is conductor? This is 1 for a solid-core wire and less than 1 for a stranded wire.
-    :param insulator_density: Density of the wire insulator [kg/m^3]
-    :param insulator_dielectric_strength: Dielectric strength of the wire insulator [V/m]. 12e6 corresponds to rubber.
-    :return: Mass of the wire [kg]
+
+    Args:
+        wire_length (float): Length of the wire [m]
+
+        max_current (float): Max current of the wire [Amps]
+
+        allowable_voltage_drop (float): How much is the voltage allowed to drop along the wire?
+
+        material (str): Conductive material of the wire ("aluminum"). Determines density and resistivity. One of:
+
+            * "sodium"
+
+            * "lithium"
+
+            * "calcium"
+
+            * "potassium"
+
+            * "beryllium"
+
+            * "aluminum"
+
+            * "magnesium"
+
+            * "copper"
+
+            * "silver"
+
+            * "gold"
+
+            * "iron"
+
+        insulated (bool): Should we add the mass of the wire's insulator coating? Usually you'll want to leave this True.
+
+        max_voltage (float): Maximum allowable voltage (used for sizing insulator). 600 is a common off-the-shelf rating.
+
+        wire_packing_factor (float): What fraction of the enclosed cross section is conductor? This is 1 for solid wire,
+            and less for stranded wire.
+
+        insulator_density (float): Density of the insulator [kg/m^3]
+
+        insulator_dielectric_strength (float): Dielectric strength of the insulator [V/m]. The default value of 12e6 corresponds
+        to rubber.
+
+        insulator_min_thickness (float): Minimum thickness of the insulator [m]. This is essentially a gauge limit.
+        The default value is 0.2 mm.
+
+        return_dict (bool): If True, returns a dictionary of all local variables. If False, just returns the wire
+        mass as a float. Defaults to False.
+
+
+    Returns: If `return_dict` is False (default), returns the wire mass as a single number. If `return_dict` is True,
+    returns a dictionary of all local variables.
     """
     if material == "sodium":  # highly reactive with water & oxygen, low physical strength
         density = 970  # kg/m^3
@@ -225,7 +338,11 @@ def mass_wires(
 
     # Insulator mass
     if insulated:
-        insulator_thickness = max_voltage / insulator_dielectric_strength
+        insulator_thickness = np.softmax(
+            4.0 * max_voltage / insulator_dielectric_strength,
+            insulator_min_thickness,
+            softness=0.005 * u.inch,
+        )
         radius_conductor = (area_conductor / wire_packing_factor / np.pi) ** 0.5
         radius_insulator = radius_conductor + insulator_thickness
         area_insulator = np.pi * radius_insulator ** 2 - area_conductor
@@ -235,7 +352,12 @@ def mass_wires(
         mass_insulator = 0
 
     # Total them up
-    return mass_conductor + mass_insulator
+    mass_total = mass_conductor + mass_insulator
+
+    if return_dict:
+        return locals()
+    else:
+        return mass_total
 
 
 if __name__ == '__main__':
