@@ -1,9 +1,10 @@
-from typing import Union, List, Dict, Callable, Any
+from typing import Union, List, Dict, Callable, Any, Tuple, Set, Optional
 import json
 import casadi as cas
 import aerosandbox.numpy as np
 from aerosandbox.tools import inspect_tools
 from sortedcontainers import SortedDict
+import copy
 
 
 class Opti(cas.Opti):
@@ -317,7 +318,7 @@ class Opti(cas.Opti):
 
         # Apply bounds
         if not (freeze and self.ignore_violated_parametric_constraints):
-            if not log_transform:
+            if (not log_transform) or (freeze):
                 if lower_bound is not None:
                     self.subject_to(
                         var / scale >= lower_bound / scale,
@@ -472,7 +473,7 @@ class Opti(cas.Opti):
                     >>> vector_param = opti.parameter(value=np.linspace(0, 5, 10)) # Initializes a vector parameter of
                     >>> # length 10, with all 10 elements set to a value varying from 0 to 5.
 
-            n_vars: [Optional] Used to manually override the dimensionality of the parameter to create; if not
+            n_params: [Optional] Used to manually override the dimensionality of the parameter to create; if not
             provided, the dimensionality of the parameter is inferred from `value`.
 
                 The only real case where you need to use this argument would be if you are initializing a vector
@@ -480,7 +481,7 @@ class Opti(cas.Opti):
                 For example:
 
                     >>> opti = asb.Opti()
-                    >>> vector_var = opti.parameter(value=5, n_vars=10) # Initializes a vector parameter of length
+                    >>> vector_param = opti.parameter(value=5, n_params=10) # Initializes a vector parameter of length
                     >>> # 10, with all 10 elements set to a value of 5.
 
         Returns:
@@ -507,7 +508,8 @@ class Opti(cas.Opti):
               verbose: bool = True,
               jit: bool = False,  # TODO document, add unit tests for jit
               options: Dict = None,  # TODO document
-              ) -> cas.OptiSol:
+              behavior_on_failure: str = "raise",
+              ) -> "OptiSol":
         """
         Solve the optimization problem using CasADi with IPOPT backend.
 
@@ -539,18 +541,29 @@ class Opti(cas.Opti):
                 quantities of optimization variables (e.g. for plotting), use the `Opti.debug.value(x)` syntax for
                 each variable `x`.
 
-            verbose: Should we print the output of IPOPT?
+            verbose: Controls the verbosity of the solver. If True, IPOPT will print its progress to the console.
 
-            jit: # TODO
+            jit: Experimental. If True, the optimization problem will be compiled to C++ and then JIT-compiled
+                using the CasADi JIT compiler. This can lead to significant speedups, but may also lead to
+                unexpected behavior, and may not work on all platforms.
 
-            options: # TODO
+            options: [Optional] A dictionary of options to pass to IPOPT. See the IPOPT documentation for a list of
+                available options.
+
+            behavior_on_failure: [Optional] What should we do if the optimization fails? Options are:
+
+                * "raise": Raise an exception. This is the default behavior.
+
+                * "return_last": Returns the solution from the last iteration, and raise a warning.
+
+                    NOTE: The returned solution may not be feasible! (It also may not be optimal.)
 
         Returns: An OptiSol object that contains the solved optimization problem. To extract values, use
-            OptiSol.value(variable).
+            my_optisol(variable).
 
             Example:
                 >>> sol = opti.solve()
-                >>> x_opt = sol.value(x) # Get the value of variable x at the optimum.
+                >>> x_opt = sol(x) # Get the value of variable x at the optimum.
 
         """
         if parameter_mapping is None:
@@ -627,18 +640,137 @@ class Opti(cas.Opti):
             self.callback(callback)
 
         # Do the actual solve
-        sol = super().solve()
+        if behavior_on_failure == "raise":
+            sol = OptiSol(
+                opti=self,
+                cas_optisol=super().solve()
+            )
+        elif behavior_on_failure == "return_last":
+            try:
+                sol = OptiSol(
+                    opti=self,
+                    cas_optisol=super().solve()
+                )
+            except RuntimeError:
+                import warnings
+                warnings.warn("Optimization failed. Returning last solution.")
+
+                sol = OptiSol(
+                    opti=self,
+                    cas_optisol=self.debug
+                )
 
         if self.save_to_cache_on_solve:
             self.save_solution()
 
         return sol
 
+    def solve_sweep(self,
+                    parameter_mapping: Dict[cas.MX, np.ndarray] = None,
+                    update_initial_guesses_between_solves=False,
+                    verbose=True,
+                    solve_kwargs: Dict = None,
+                    return_callable: bool = False,
+                    ) -> Union[np.ndarray, Callable[[cas.MX], np.ndarray]]:
+
+        # Handle defaults
+        if solve_kwargs is None:
+            solve_kwargs = {}
+        solve_kwargs = {
+            **dict(
+                verbose=False,
+            ),
+            **solve_kwargs
+        }
+
+        # Split parameter_mappings up so that it can be passed into run() via np.vectorize
+        keys: Tuple[cas.MX] = tuple(parameter_mapping.keys())
+        values: Tuple[np.ndarray[float]] = tuple(parameter_mapping.values())
+
+        # Display an output
+        if verbose:
+            print("Running optimization sweep in serial...")
+
+        n_runs = np.broadcast(*values).size
+        run_number = 1
+
+        def run(*args: Tuple[float]) -> Optional["OptiSol"]:
+            # Reconstruct parameter mapping on a run-by-run basis by zipping together keys and this run's values.
+            parameter_mappings_for_this_run: [cas.MX, float] = {
+                k: v
+                for k, v in zip(keys, args)
+            }
+
+            # Pull in run_number so that we can increment this counter
+            nonlocal run_number
+
+            # Display as needed
+            if verbose:
+                print(
+                    "|".join(
+                        [
+                            f"Run {run_number}/{n_runs}".ljust(12)
+                        ] + [
+                            f"{v:10.5g}"
+                            for v in args
+                        ] + [""]
+                    ),
+                    end=''  # Leave the newline off, since we'll complete the line later with a success or fail print.
+                )
+
+            run_number += 1
+
+            import time
+            start_time = time.time()
+
+            try:
+                sol = self.solve(
+                    parameter_mapping=parameter_mappings_for_this_run,
+                    **solve_kwargs
+                )
+
+                if update_initial_guesses_between_solves:
+                    self.set_initial_from_sol(sol)
+
+                if verbose:
+                    stats = sol.stats()
+                    print(f" Solved in {stats['iter_count']} iterations, {time.time() - start_time:.2f} sec.")
+
+                return sol
+
+            except RuntimeError:
+                if verbose:
+                    sol = OptiSol(opti=self, cas_optisol=self.debug)
+                    stats = sol.stats()
+                    print(f" Failed in {stats['iter_count']} iterations, {time.time() - start_time:.2f} sec.")
+
+                return None
+
+        run_vectorized = np.vectorize(
+            run,
+            otypes='O'  # object output
+        )
+
+        sols = run_vectorized(*values)
+
+        if return_callable:
+
+            def get_vals(x: cas.MX) -> np.ndarray:
+                return np.vectorize(
+                    lambda sol: sol.value(x) if sol is not None else np.nan
+                )(sols)
+
+            return get_vals
+
+        else:
+            return sols
+
     ### Debugging Methods
     def find_variable_declaration(self,
                                   index: int,
-                                  use_full_filename: bool = False
-                                  ):
+                                  use_full_filename: bool = False,
+                                  return_string: bool = False,
+                                  ) -> Union[None, str]:
         ### Check inputs
         if index < 0:
             raise ValueError("Indices must be nonnegative.")
@@ -659,7 +791,7 @@ class Opti(cas.Opti):
         title = f"{'Scalar' if is_scalar else 'Vector'} variable"
         if not is_scalar:
             title += f" (index {index - index_of_first_element} of {n_vars})"
-        print("\n".join([
+        string = "\n".join([
             "",
             f"{title} defined in `{str(filename) if use_full_filename else filename.name}`, line {lineno}:",
             "",
@@ -667,12 +799,17 @@ class Opti(cas.Opti):
             source,
             "```"
         ])
-        )
+
+        if return_string:
+            return string
+        else:
+            print(string)
 
     def find_constraint_declaration(self,
                                     index: int,
-                                    use_full_filename: bool = False
-                                    ):
+                                    use_full_filename: bool = False,
+                                    return_string: bool = False
+                                    ) -> Union[None, str]:
         ### Check inputs
         if index < 0:
             raise ValueError("Indices must be nonnegative.")
@@ -695,7 +832,7 @@ class Opti(cas.Opti):
         title = f"{'Scalar' if is_scalar else 'Vector'} constraint"
         if not is_scalar:
             title += f" (index {index - index_of_first_element} of {n_cons})"
-        print("\n".join([
+        string = "\n".join([
             "",
             f"{title} defined in `{str(filename) if use_full_filename else filename.name}`, line {lineno}:",
             "",
@@ -703,7 +840,11 @@ class Opti(cas.Opti):
             source,
             "```"
         ])
-        )
+
+        if return_string:
+            return string
+        else:
+            print(string)
 
     ### Advanced Methods
 
@@ -1019,7 +1160,228 @@ class Opti(cas.Opti):
             raise ValueError("Bad value of `method`!")
 
 
+class OptiSol:
+    def __init__(self,
+                 opti: Opti,
+                 cas_optisol: cas.OptiSol
+                 ):
+        """
+        An OptiSol object represents a solution to an optimization problem. This class is a wrapper around CasADi's
+        `OptiSol` class that provides convenient solution query utilities for various Python data types.
+
+        Args:
+            opti: The `Opti` object that generated this solution.
+
+            cas_optisol: The `casadi.OptiSol` object from CasADi's optimization solver.
+
+        Returns:
+            An `OptiSol` object.
+
+        Usage:
+            >>> # Initialize an Opti object.
+            >>> opti = asb.Opti()
+            >>>
+            >>> # Define a scalar variable.
+            >>> x = opti.variable(init_guess=2.0)
+            >>>
+            >>> # Define an objective function.
+            >>> opti.minimize(x ** 2)
+            >>>
+            >>> # Solve the optimization problem. `sol` is now a
+            >>> sol = opti.solve()
+            >>>
+            >>> # Retrieve the value of the variable x in the solution:
+            >>> x_value = sol.value(x)
+            >>>
+            >>> # Or, to be more concise:
+            >>> x_value = sol(x)
+        """
+        self.opti = opti
+        self._sol = cas_optisol
+
+    def __call__(self, x: Union[cas.MX, np.ndarray, float, int, List, Tuple, Set, Dict, Any]) -> Any:
+        """
+        A shorthand alias for `sol.value(x)`. See `OptiSol.value()` documentation for details.
+
+        Args:
+            x: A Python data structure to substitute values into, using the solution in this OptiSol object.
+
+        Returns:
+
+            A copy of `x`, where all symbolic optimization variables (recursively substituted at unlimited depth)
+            have been converted to float or array values.
+
+        """
+        return self.value(x)
+
+    def value(self,
+              x: Union[cas.MX, np.ndarray, float, int, List, Tuple, Set, Dict, Any],
+              recursive: bool = True,
+              warn_on_unknown_types: bool = False
+              ) -> Any:
+        """
+        At its simplest: converts a symbolic optimization variable to a concrete float or array value.
+
+        More generally: converts any Python data structure (along with any of its contents, recursively, at unlimited
+        depth), replacing any symbolic optimization variables it finds with concrete float or array values.
+
+        Args:
+            x: A Python data structure to substitute values into, using the solution in this OptiSol object.
+
+            recursive: If True, the substitution will be performed recursively. Otherwise, only the top-level data
+                structure will be converted.
+
+            warn_on_unknown_types: If True, a warning will be issued if a data type that cannot be converted or
+            parsed as definitively un-convertable is encountered.
+
+        Returns:
+            A copy of `x`, where all symbolic optimization variables (recursively substituted at unlimited depth)
+            have been converted to float or array values.
+
+        Usage:
+
+
+
+        """
+        if not recursive:
+            return self._sol.value(x)
+
+        # If it's a CasADi type, do the conversion, and call it a day.
+        if np.is_casadi_type(x, recursive=False):
+            return self._sol.value(x)
+
+        t = type(x)
+
+        # If it's a Python iterable, recursively convert it, and preserve the type as best as possible.
+        if issubclass(t, list):
+            return [self.value(i) for i in x]
+        if issubclass(t, tuple):
+            return tuple([self.value(i) for i in x])
+        if issubclass(t, (set, frozenset)):
+            return {self.value(i) for i in x}
+        if issubclass(t, dict):
+            return {
+                self.value(k): self.value(v)
+                for k, v in x.items()
+            }
+
+        # Skip certain Python types
+        if issubclass(t, (
+                bool, str,
+                int, float, complex,
+                range,
+                type(None),
+                bytes, bytearray, memoryview
+        )):
+            return x
+
+        # Skip certain CasADi types
+        if issubclass(t, (
+                cas.Opti, cas.OptiSol
+        )):
+            return x
+
+        # If it's any other type, try converting its attribute dictionary, if it has one:
+        try:
+            new_x = copy.copy(x)
+
+            for k, v in x.__dict__.items():
+                setattr(new_x, k, self.value(v))
+
+            return new_x
+
+        except AttributeError:
+            pass
+
+        # Try converting it blindly. This will catch most NumPy-array-like types.
+        try:
+            return self._sol.value(x)
+        except (NotImplementedError, TypeError, ValueError):
+            pass
+
+        # At this point, we're not really sure what type the object is. Raise a warning if directed and return the
+        # item, then hope for the best.
+        if warn_on_unknown_types:
+            import warnings
+            warnings.warn(f"In solution substitution, could not convert an object of type {t}.\n"
+                          f"Returning it and hoping for the best.", UserWarning)
+
+        return x
+
+    def stats(self) -> Dict[str, Any]:
+        return self._sol.stats()
+
+    def value_variables(self):
+        return self._sol.value_variables()
+
+    def value_parameters(self):
+        return self._sol.value_parameters()
+
+    def show_infeasibilities(self, tol: float = 1e-3) -> None:
+        """
+        Prints a summary of any violated constraints in the solution.
+
+        Args:
+
+            tol: The tolerance for violation. If the constraint is violated by less than this amount, it will not be
+                printed.
+
+        Returns: None (prints to console)
+        """
+        lbg = self(self.opti.lbg)
+        ubg = self(self.opti.ubg)
+
+        g = self(self.opti.g)
+
+        constraint_violated = np.logical_or(
+            g + tol < lbg,
+            g - tol > ubg
+        )
+
+        lbg_isfinite = np.isfinite(lbg)
+        ubg_isfinite = np.isfinite(ubg)
+
+        for i in np.arange(len(g)):
+            if constraint_violated[i]:
+                print("-" * 50)
+
+                if lbg_isfinite[i] and ubg_isfinite[i]:
+                    if lbg[i] == ubg[i]:
+                        print(f"{lbg[i]} == {g[i]} (violation: {np.abs(g[i] - lbg[i])})")
+                    else:
+                        print(f"{lbg[i]} < {g[i]} < {ubg[i]} (violation: {np.maximum(lbg[i] - g[i], g[i] - ubg[i])})")
+                elif lbg_isfinite[i] and not ubg_isfinite[i]:
+                    print(f"{lbg[i]} < {g[i]} (violation: {lbg[i] - g[i]})")
+                elif not lbg_isfinite[i] and ubg_isfinite[i]:
+                    print(f"{g[i]} < {ubg[i]} (violation: {g[i] - ubg[i]})")
+                else:
+                    raise ValueError(
+                        "Contact the AeroSandbox developers if you see this message; it should be impossible.")
+
+                self.opti.find_constraint_declaration(index=i)
+
+
 if __name__ == '__main__':
     import pytest
 
-    pytest.main()
+    # pytest.main()
+
+
+    opti = Opti()  # set up an optimization environment
+
+    a = opti.parameter(1)
+    b = opti.parameter(100)
+
+    # Define optimization variables
+    x = opti.variable(init_guess=0)
+    y = opti.variable(init_guess=0)
+
+    # Define objective
+    f = (a - x) ** 2 + b * (y - x ** 2) ** 2
+    opti.minimize(f)
+
+    # Optimize
+    sol = opti.solve()
+
+    for i in [x, y]:
+        assert sol.value(i) == pytest.approx(1, abs=1e-4)
