@@ -17,19 +17,27 @@ class OperatingPoint(AeroSandboxObject):
                  r: float = 0.,
                  ):
         """
-        An object that represents the instantaneous flight conditions of an aircraft.
+        An object that represents the instantaneous aerodynamic flight conditions of an aircraft.
 
         Args:
-            atmosphere:
-            velocity: The flight velocity, expressed in true airspeed. [m/s]
+            atmosphere: The atmosphere object (of type asb.Atmosphere). Defaults to sea level conditions.
+
+            velocity: The flight velocity, expressed as a true airspeed. [m/s]
+
             alpha: The angle of attack. [degrees]
-            beta: The sideslip angle. (Reminder: convention that a positive beta implies that the oncoming air comes from the pilot's right-hand side.) [degrees]
+
+            beta: The sideslip angle. (Reminder: convention that a positive beta implies that the oncoming air comes
+            from the pilot's right-hand side.) [degrees]
+
             p: The roll rate about the x_b axis. [rad/sec]
+
             q: The pitch rate about the y_b axis. [rad/sec]
+
             r: The yaw rate about the z_b axis. [rad/sec]
+
         """
         self.atmosphere = atmosphere
-        self.velocity = velocity  # TODO rename "airspeed"?
+        self.velocity = velocity
         self.alpha = alpha
         self.beta = beta
         self.p = p
@@ -162,7 +170,7 @@ class OperatingPoint(AeroSandboxObject):
 
         def makeline(k, v):
             name = trim_string(str(k).strip(), length=10).rjust(10)
-            item = trim_string(str(v).strip(), length=40).ljust(40)
+            item = trim_string(str(v).strip(), length=120).ljust(120)
 
             line = f"{name}: {item}"
 
@@ -303,6 +311,16 @@ class OperatingPoint(AeroSandboxObject):
             self.atmosphere.density() / Atmosphere(altitude=0, method="isa").density()
         )
 
+    def energy_altitude(self):
+        """
+        Returns the energy altitude associated with the current flight condition, in meters.
+
+        The energy altitude is the altitude at which a stationary aircraft would have the same total energy (kinetic
+        + gravitational potential) as the aircraft at the current flight condition.
+
+        """
+        return self.atmosphere.altitude + 1 / (2 * 9.81) * self.velocity ** 2
+
     def convert_axes(self,
                      x_from: Union[float, np.ndarray],
                      y_from: Union[float, np.ndarray],
