@@ -1,7 +1,8 @@
 import inspect
-from typing import List, Union
+from typing import List, Union, Tuple, Optional, Set, Any
 from pathlib import Path
 from aerosandbox.tools.string_formatting import has_balanced_parentheses
+import numpy as np
 
 
 def get_caller_source_location(
@@ -299,6 +300,192 @@ def get_function_argument_names_from_source_code(source_code: str) -> List[str]:
     return arg_names
 
 
+def codegen(
+        x: Any,
+        indent_str: str = "    ",
+        _required_imports: Optional[Set[str]] = None,
+        _recursion_depth: int = 0,
+) -> Tuple[str, Set[str]]:
+    """
+    Attempts to generate a string of Python code that, when evaluated, would produce the same value as the input.
+    Also generates the required imports for the code to run.
+
+    In other words, in general, the following should evaluate True:
+
+    >>> code, imports = codegen(x)
+    >>> for import_str in imports:
+    >>>     exec(import_str)
+    >>> eval(code) == x  # Should evaluate True
+
+    Not guaranteed to work for all inputs, but should work for most common cases.
+
+    Args:
+
+        x: The object to generate the code of.
+
+        indent_str: The string to use for indentation. Defaults to four spaces.
+
+        _required_imports: A set of strings containing the names of all required imports. This is an internal
+        argument that should not be used by the user.
+
+        _recursion_depth: The current recursion depth. This is an internal argument that should not be used by the user.
+
+    Returns: A tuple containing:
+
+        - The string of Python code that, when evaluated, would produce the same value as the input.
+
+        - A set of strings that, when evaluated, would import all of the required imports for the code to run.
+
+    Examples:
+
+        >>> codegen(5)
+        ('5', set())
+
+        >>> codegen([1, 2, 3])
+        ('[1, 2, 3]', set())
+
+        >>> codegen(np.array([1, 2, 3]))
+        ('np.array([1, 2, 3])', {'import numpy as np'})
+
+        >>> codegen(dict(my_int=4, my_array=np.array([1, 2, 3])))
+        ('{
+        	'my_int': 4,
+        	'my_array': np.array([1, 2, 3]),
+        }', {'import numpy as np'})
+
+    """
+    ### Set defaults
+    if _required_imports is None:
+        _required_imports = set()
+
+    import_aliases = {
+        "aerosandbox"      : "asb",
+        "aerosandbox.numpy": "np",
+        "numpy"            : "np",
+    }
+
+    indent = indent_str * _recursion_depth
+    next_indent = indent_str * (_recursion_depth + 1)
+
+    if isinstance(x, (
+            bool, str,
+            int, float, complex,
+            range,
+            type(None),
+            bytes, bytearray, memoryview
+    )):
+        code = repr(x)
+
+    elif isinstance(x, list):
+        if len(x) == 0:
+            code = "[]"
+        else:
+            lines = []
+            lines.append("[")
+            for xi in x:
+                item_code, item_required_imports = codegen(xi, _recursion_depth=_recursion_depth + 1)
+
+                _required_imports.update(item_required_imports)
+
+                lines.append(next_indent + item_code + ",")
+            lines.append(indent + "]")
+            code = "\n".join(lines)
+
+    elif isinstance(x, tuple):
+        if len(x) == 0:
+            code = "()"
+        else:
+            lines = []
+            lines.append("(")
+            for xi in x:
+                item_code, item_required_imports = codegen(xi, _recursion_depth=_recursion_depth + 1)
+
+                _required_imports.update(item_required_imports)
+
+                lines.append(next_indent + item_code + ",")
+            lines.append(indent + ")")
+            code = "\n".join(lines)
+
+    elif isinstance(x, (set, frozenset)):
+        if len(x) == 0:
+            code = "set()"
+        else:
+            lines = []
+            lines.append("{")
+            for xi in x:
+                item_code, item_required_imports = codegen(xi, _recursion_depth=_recursion_depth + 1)
+
+                _required_imports.update(item_required_imports)
+
+                lines.append(next_indent + item_code + ",")
+            lines.append(indent + "}")
+            code = "\n".join(lines)
+
+    elif isinstance(x, dict):
+        if len(x) == 0:
+            code = "{}"
+        else:
+            lines = []
+            lines.append("{")
+            for k, v in x.items():
+                k_code, k_required_imports = codegen(k, _recursion_depth=_recursion_depth + 1)
+                v_code, v_required_imports = codegen(v, _recursion_depth=_recursion_depth + 1)
+
+                _required_imports.update(k_required_imports)
+                _required_imports.update(v_required_imports)
+
+                lines.append(next_indent + k_code + ": " + v_code + ",")
+            lines.append(indent + "}")
+            code = "\n".join(lines)
+
+    elif isinstance(x, np.ndarray):
+        _required_imports.add("import numpy as np")
+        code = f"np.{repr(x)}"
+
+    else:  # At this point, we assume it's a class instance, and could be from any package.
+
+        module_name = x.__class__.__module__
+        package_name = module_name.split(".")[0]
+
+        if package_name == "builtins":
+            pre_string = ""
+        # elif package_name in import_aliases:
+        #     pre_string = import_aliases[package_name] + "."
+        else:
+            _required_imports.add(
+                f"from {module_name} import {x.__class__.__name__}"
+            )
+
+        lines = []
+        lines.append(x.__class__.__name__ + "(")
+        for arg_name in inspect.getfullargspec(x.__init__).args[1:]:
+            if hasattr(x, arg_name):
+                arg_value = getattr(x, arg_name)
+
+                if inspect.ismethod(arg_value) or inspect.isfunction(arg_value):
+                    continue
+
+                arg_code, arg_required_imports = codegen(arg_value, _recursion_depth=_recursion_depth + 1)
+
+                _required_imports.update(arg_required_imports)
+
+                lines.append(next_indent + arg_name + "=" + arg_code + ",")
+        lines.append(indent + ")")
+        code = "\n".join(lines)
+
+    return code, _required_imports
+    #
+    # if _recursion_depth == 0:
+    #     if len(_required_imports) > 0:
+    #         imports = "\n".join(sorted(_required_imports))
+    #         return imports + "\n\n" + code
+    #
+    #     else:
+    #         return code
+    # else:
+    #     return code, _required_imports
+
+
 if __name__ == '__main__':
     def dashes():
         """A quick macro for drawing some dashes, to make the terminal output clearer to distinguish."""
@@ -337,3 +524,21 @@ if __name__ == '__main__':
     )
 
     location = get_caller_source_location()
+
+    dashes()
+
+    print("Codegen test:")
+
+
+    def pc(x):
+        code, imports = codegen(x)
+        print("\n".join(sorted(imports)))
+        print(code + "\n" + "-" * 50)
+
+
+    pc(1)
+    pc([1, 2, 3])
+    pc([1, 2, [3, 4, 5], 6])
+    pc({"a": 1, "b": 2})
+    pc(np.array([1, 2, 3]))
+    pc(dict(myarray=np.array([1, 2, 3]), yourarray=np.arange(10)))
