@@ -21,7 +21,7 @@ minute = 60
 hour = 60 * minute
 day = 24 * hour
 week = 7 * day
-year = 365.25 * day
+year = 365.2425 * day
 month = year / 12
 
 # Force (equivalents in Newtons)
@@ -29,9 +29,10 @@ newton = 1
 lbf = slug * foot
 
 # Speed (equivalents in m/s)
-knot = 1.852 * 1000 / hour
+kph = 1000 * meter / hour
+knot = 1.852 * kph
 mph = mile / hour
-fps = foot / 1
+fps = foot / second
 
 # Volume (equivalents in m^3)
 liter = 0.001
@@ -42,9 +43,22 @@ quart = gallon_us / 4
 
 # Pressure (equivalents in Pa)
 pascal = 1
-torr = 133.3223684211
 atm = 101325
+torr = atm / 760
 psi = lbf / inch ** 2
+psf = lbf / foot ** 2
+
+# Power (equivalents in Watts)
+watt = 1
+horsepower = 550 * foot * lbf / second
+hp = horsepower
+
+# Energy (equivalents in Joules)
+joule = 1
+btu = 1055.05585262
+calorie = 4.184
+kcal = 1000 * calorie
+watt_hour = watt * hour
 
 # SI prefixes
 yocto = 1e-24
